<?php
/**
 * Whups_Driver Class.
 *
 * ABC for backends.
 *
 * TODO: Needs updating to include method stubs for all required
 * methods (just to indicate what methods need to be implemented by
 * other backends).
 *
 * $Horde: whups/lib/Driver.php,v 1.174 2008/02/21 14:34:49 jan Exp $
 *
 * Copyright 2001-2007 Robert E. Coyle <robertecoyle@hotmail.com>
 *
 * See the enclosed file LICENSE for license information (BSD). If you
 * did not receive this file, see http://www.horde.org/licenses/bsdl.php.
 *
 * @author  Robert E. Coyle <robertecoyle@hotmail.com>
 * @package Whups
 */
class Whups_Driver {

    /**
     * @var array
     */
    var $_params;

    /**
     */
    function Whups_Driver($params)
    {
        $this->_params = $params;
    }

    /**
     */
    function setAttributes($info, &$ticket)
    {
        $ticket_id = $ticket->getId();

        foreach ($info as $name => $value) {
            if (substr($name, 0, 10) == 'attribute_' &&
                $ticket->get($name) != $value) {
                $attribute_id = (int)substr($name, 10);
                $ticket->change($name, $value);
                $this->_setAttributeValue($ticket_id, $attribute_id, $value);
                $this->updateLog($ticket_id, Auth::getAuth(), array('attribute' => $attribute_id . ':' . $value));
            }
        }
    }

    /**
     * @param integer $ticket_id
     */
    function getHistory($ticket_id)
    {
        $rows = $this->_getHistory($ticket_id);
        if (is_a($rows, 'PEAR_Error')) {
            return $rows;
        }

        $history = array();
        foreach ($rows as $row) {
            $transaction = $row['transaction_id'];

            $history[$transaction]['timestamp'] = $row['timestamp'];
            $history[$transaction]['user_id'] = $row['user_id'];
            $history[$transaction]['ticket_id'] = $row['ticket_id'];

            switch ($row['log_type']) {
            case 'comment':
                $history[$transaction]['comment'] = $row['comment_text'];
                $history[$transaction]['changes'][] = array('type' => $row['log_type'],
                                                            'value' => $row['log_value'],
                                                            'comment' => $row['comment_text']);
                break;

            default:
                $label = null;
                $value = $row['log_value'];
                switch ($row['log_type']) {
                case 'queue':
                    $label = $row['queue_name'];
                    break;

                case 'type':
                    $label = $row['type_name'];
                    break;

                case 'state':
                    $label = $row['state_name'];
                    break;

                case 'priority':
                    $label = $row['priority_name'];
                    break;

                case 'attribute':
                    $label = $row['attribute_name'];
                    $value = substr($value, strpos($value, ':') + 1);
                    break;

                case 'due':
                    $label = $row['log_value_num'];
                    break;
                }

                $history[$transaction]['changes'][] = array(
                    'type' => $row['log_type'],
                    'value' => $value,
                    'label' => $label);
            }
        }

        return $history;
    }

    /**
     */
    function getQueue($queueId)
    {
        return $GLOBALS['registry']->call('tickets/getQueueDetails', array($queueId));
    }

    /**
     */
    function getQueues()
    {
        return $GLOBALS['registry']->call('tickets/listQueues');
    }

    /**
     */
    function getVersionInfo($queue)
    {
        return $GLOBALS['registry']->call('tickets/listVersions', array($queue));
    }

    /**
     */
    function getVersions($queue)
    {
        if (empty($queue)) {
            return array();
        }

        $versioninfo = $this->getVersionInfo($queue);
        if (is_a($versioninfo, 'PEAR_Error')) {
            return $versioninfo;
        }

        $versions = array();
        foreach ($versioninfo as $vinfo) {
            $versions[$vinfo['id']] = $vinfo['name'];
            if (!empty($vinfo['description'])) {
                $versions[$vinfo['id']] .= ': ' . $vinfo['description'];
            }
        }

        return $versions;
    }

    /**
     */
    function getVersion($version)
    {
        return $GLOBALS['registry']->call('tickets/getVersionDetails', array($version));
    }

    /**
     */
    function getCategories()
    {
        return array('unconfirmed' => _("Unconfirmed"),
                     'new' => _("New"),
                     'assigned' => _("Assigned"),
                     'resolved' => _("Resolved"));
    }

    /**
     * Function to fetch the attributes for a specific ticket type. Will check
     * first if external attributes need to be fetched from hooks or whether
     * to use the standard ones defined within Whups.
     *
     * @params string $type  The ticket type.
     *
     * @return Array of attributes.
     */
    function getAttributesForType($type = null)
    {
        /* Check if we have external hooks defined for attributes. */
        if (!empty($GLOBALS['conf']['hooks']['attributes'])) {
            /* External hooks required, call the whups hook function. */
            return Horde::callHook('_whups_hook_get_attributes', $type, 'whups');
        } else {
            /* Do standard attributes instead, create full definition array
             * but keep them all as text fields. */
            $attributes = $this->_getAttributesForType($type);
            foreach (array_keys($attributes) as $id) {
                $name = $attributes[$id];
                $attributes[$id] = array('human_name' => $name,
                                         'type'       => 'text',
                                         'required'   => false,
                                         'readonly'   => false,
                                         'desc'       => null,
                                         'params'     => null);
            }
            return $attributes;
        }
    }

    /**
     */
    function getAllTicketAttributesWithNames($ticket_id)
    {
        if (!empty($GLOBALS['conf']['hooks']['attributes'])) {
            return Horde::callHook('_whups_hook_get_ticket_attributes', $ticket_id,'whups');
        } else {
            $attributes = array();
            $ta = $this->_getAllTicketAttributesWithNames($ticket_id);
            foreach ($ta as $a) {
                $attributes[$a['attribute_id']] = array(
                    'id'         => $a['attribute_id'],
                    'human_name' => $a['attribute_name'],
                    'type'       => 'text',
                    'required'   => false,
                    'readonly'   => false,
                    'desc'       => null,
                    'params'     => null,
                    'value'      => $a['attribute_value'],
                );
            }
            return $attributes;
        }
    }

    /**
     * Base deleteQueue() implementation. Should be called by driver
     * subclasses after successful removal from the backend. Takes
     * care of cleaning up queue permissions.
     *
     * @param integer $queueId  The id of the queue being deleted.
     */
    function deleteQueue($queueId)
    {
        $perm = &$GLOBALS['perms']->getPermission("whups:queues:$queueId");
        if (is_a($perm, 'PEAR_Error')) {
            return $perm;
        }

        return $perm->delete();
    }

    /**
     */
    function filterTicketsByState($tickets, $state_category = array())
    {
        /* Take a list of tickets and return only those of the
         * specified state_category. */
        $tickets_filtered = array();
        foreach ($tickets as $ticket) {
            foreach ($state_category as $state) {
                if ($ticket['state_category'] == $state) {
                    $tickets_filtered[] = $ticket;
                }
            }
        }

        return $tickets_filtered;
    }

    /**
     * We do some ugly work in here to make sure that no one gets
     * comments mailed to them that they shouldn't see (because of
     * group permissions).
     */
    function mail($ticket_id, $recipients, $subject, $message, $from, $reminder = false)
    {
        global $conf, $registry, $prefs;

        require_once 'Mail.php';
        require_once 'Horde/MIME.php';
        require_once 'Horde/MIME/Headers.php';

        /* Set up recipients and message headers. */
        if (!is_array($recipients)) {
            $recipients = array($recipients);
        }

        $msg_headers = new MIME_Headers();
        $msg_headers->addMessageIdHeader();
        $msg_headers->addAgentHeader();
        $msg_headers->addHeader('Date', date('r'));
        $msg_headers->addHeader('X-Whups-Generated', 1);

        $mail_always = null;
        if (!$reminder && !empty($conf['mail']['always_copy'])) {
            $mail_always = $conf['mail']['always_copy'];
            if (strpos($mail_always, '<@>') !== false) {
                $ticket = Whups_Ticket::makeTicket($ticket_id);
                $mail_always = str_replace('<@>', $ticket->get('queue_name'), $mail_always);
            }
            $msg_headers->addHeader('To', $mail_always);
            $recipients[] = $mail_always;
        } else {
            $msg_headers->addHeader('To', 'Ticket_Listeners:;');
        }

        if (!empty($conf['mail']['from_addr'])) {
            $msg_headers->addHeader('From', $conf['mail']['from_addr']);
        } else {
            $msg_headers->addHeader('From', Whups::formatUser($from));
        }

        $msg_headers->addHeader('Subject', (is_null($ticket_id) ? '' : '[' . $registry->get('name') . ' #' . $ticket_id . '] ') . $subject);

        /* Get our array of comments, sorted in the appropriate
         * order. */
        if (!is_null($ticket_id)) {
            $comments = $this->getHistory($ticket_id);
            if ($conf['mail']['commenthistory'] == 'new' && count($comments)) {
                $comments = array_pop($comments);
                $comments = array($comments);
            } elseif ($conf['mail']['commenthistory'] != 'chronological') {
                $comments = array_reverse($comments);
            }
        } else {
            $comments = array();
        }

        $mails = array();
        $messages = array();
        foreach ($recipients as $user) {
            if (($user == $from && $user == Auth::getAuth()) && $prefs->getValue('email_others_only')) {
                continue;
            }

            /* Make sure to check permissions as a guest for the 'always_copy'
             * address, and as the recipient for all others. */
            if (!empty($mail_always) && $user == $mail_always) {
                $mycomments = Whups::permissionsFilter($comments, 'comment', PERMS_READ, '');
                $key = implode('|', array_keys($mycomments));
                $messages[$key] = str_replace('@@comment_placeholder@@', $this->formatComments($mycomments), $message);
                $mails[$key][] = $mail_always;
            } else {
                $details = Whups::getUserAttributes($user);
                if (!empty($details['email'])) {
                    $email = Whups::formatUser($user);
                    $mycomments = Whups::permissionsFilter($comments, 'comment', PERMS_READ, $details['user']);
                    $key = implode('|', array_keys($mycomments));
                    $messages[$key] = str_replace('@@comment_placeholder@@', $this->formatComments($mycomments), $message);
                    $mails[$key][] = $email;
                }
            }
        }

        /* We may have no recipients due to users excluding themselves
         * from self notifies. */
        if (count($mails)) {
            require_once 'Horde/MIME/Message.php';
            foreach ($mails as $key => $addresses) {
                $msg_headers->removeHeader('Bcc');
                $msg_headers->addHeader('Bcc', implode(', ', $addresses));
                $mime = new MIME_Message();
                $body = new MIME_Part('text/plain', $messages[$key], NLS::getCharset());
                $mime->addPart($body);
                $msg_headers->addMIMEHeaders($mime);
                $result = $mime->send(implode(', ', $addresses), $msg_headers, $conf['mailer']['type'], $conf['mailer']['params']);
                if (is_a($result, 'PEAR_Error')) {
                    Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
                }
            }
        }
    }

    function formatComments($comments)
    {
        $text = '';
        foreach ($comments as $comment) {
            if (!empty($comment['comment_text'])) {
                $text .= "\n" .
                    wordwrap(sprintf(_("%s (%s) wrote:"),
                                     Whups::formatUser($comment['user_id']),
                                     Horde_Form_Type_date::getFormattedTime($comment['timestamp'], '%Y-%m-%d %H:%M', false))) .
                    "\n\n" . wordwrap($comment['comment_text']) . "\n\n\n";
            }
        }

        return $text;
    }

    /**
     * Attempts to return a concrete Whups_Driver instance based on $driver.
     *
     * @param string $driver The type of concrete Driver subclass to return.
     * @param array $params  A hash containing any additional configuration or
     *                       connection parameters a subclass might need.
     *
     * @return Whups_Driver  The newly created concrete Whups_Driver instance.
     */
    function factory($driver = null, $params = null)
    {
        if (is_null($driver)) {
            $driver = $GLOBALS['conf']['tickets']['driver'];
        }

        $driver = basename($driver);
        $class = 'Whups_Driver_' . $driver;
        if (!class_exists($class)) {
            include dirname(__FILE__) . '/Driver/' . $driver . '.php';
        }
        if (class_exists($class)) {
            if (is_null($params)) {
                $params = Horde::getDriverConfig('tickets', $driver);
            }
            return new $class($params);
        } else {
            return PEAR::raiseError(sprintf(_("No such backend \"%s\" found"), $driver));
        }
    }

}
