<?php
/**
 * $Horde: whups/lib/Forms/Admin.php,v 1.13 2008/02/28 00:56:49 jan Exp $
 *
 * Copyright 2002-2008 The Horde Project (http://www.horde.org/)
 *
 * See the enclosed file LICENSE for license information (BSD). If you
 * did not receive this file, see http://www.horde.org/licenses/bsdl.php.
 *
 * @author  Chuck Hagenbuch <chuck@horde.org>
 * @package Whups
 */

///// Queue Forms

class AddQueueForm extends Horde_Form {

    function AddQueueForm(&$vars)
    {
        parent::Horde_Form($vars, _("Add Queue"));
        $this->appendButtons(_("Add Queue"));

        $this->addVariable(_("Queue Name"), 'name', 'text', true);
        $this->addVariable(_("Queue Description"), 'description', 'text', true);
        $this->addVariable(_("Queue Slug"), 'slug', 'text', false, false,
                           sprintf(_("Slugs allows direct access to this queue's open tickets by visiting: %s. <br /> Slug names may contain only letters, numbers or the _ (underscore) character."),
                                   Horde::applicationUrl('queue/slugname', true)),
                           array('/^[a-zA-Z1-9_]*$/'));
    }

}

class EditQueueStep1Form extends Horde_Form {

    function EditQueueStep1Form(&$vars)
    {
        global $whups_driver, $registry;

        if ($registry->hasMethod('tickets/listQueues') == $registry->getApp()) {
            parent::Horde_Form($vars, _("Edit or Delete Queues"));
            $this->setButtons(array(_("Edit Queue"), _("Delete Queue")));
        } else {
            parent::Horde_Form($vars, _("Edit Queues"));
            $this->setButtons(array(_("Edit Queue")));
        }

        $queues = Whups::permissionsFilter($whups_driver->getQueues(), 'queue', PERMS_EDIT);
        if ($queues) {
            $modtype = 'enum';
            $type_params = array($queues);
        } else {
            $modtype = 'invalid';
            $type_params = array(_("There are no queues to edit"));
        }

        $this->addVariable(_("Queue Name"), 'queue', $modtype, true, false, null, $type_params);
    }

}

class EditQueueStep2Form extends Horde_Form {

    function EditQueueStep2Form(&$vars)
    {
        global $whups_driver, $registry;

        parent::Horde_Form($vars);

        $queue = $vars->get('queue');
        $info = $whups_driver->getQueue($queue);
        if (is_a($info, 'PEAR_Error')) {
            $this->addVariable(_("Invalid Queue"), 'invalid', 'invalid', true, false, null, array(_("Invalid Queue")));
            return;
        }

        $this->setTitle(sprintf(_("Edit %s"), $info['name']));
        $this->addHidden('', 'queue', 'int', true, true);

        $mname = &$this->addVariable(_("Queue Name"), 'name', 'text', true, $info['readonly']);
        $mname->setDefault($info['name']);

        $mdesc = &$this->addVariable(_("Queue Description"), 'description', 'text', true, $info['readonly']);
        $mdesc->setDefault($info['description']);

        $mslug = &$this->addVariable(_("Queue Slug"), 'slug', 'text', false, $info['readonly']);
        $mslug->setDefault($info['slug']);

        $types = $whups_driver->getAllTypes();
        $mtypes = &$this->addVariable(_("Ticket Types associated with this Queue"), 'types', 'set', true, false, null, array($types));
        $mtypes->setDefault(array_keys($whups_driver->getTypes($queue)));
        $mdefaults = &$this->addVariable(_("Default Ticket Type"), 'default', 'enum', false, false, null, array($types));
        $mdefaults->setDefault($whups_driver->getDefaultType($queue));

        /* Versioned and version link. */
        $mversioned = &$this->addVariable(_("Keep a set of versions for this queue?"), 'versioned', 'boolean', false, $info['readonly']);
        $mversioned->setDefault($info['versioned']);
        if ($registry->hasMethod('tickets/listVersions') == $registry->getApp()) {
            $versionlink = array('text' => _("Edit the versions for this queue"), 'url' => Util::addParameter(Horde::applicationUrl('admin/?formname=editversionstep1form'), 'queue', $queue));
            $this->addVariable('', 'link', 'link', false, true, null, array($versionlink));
        }

        /* Usertype and usertype link. */
        $users = $whups_driver->getQueueUsers($queue);
        $f_users = array();
        foreach ($users as $user) {
            $f_users[$user] = Whups::formatUser($user);
        }
        asort($f_users);
        $musers = &$this->addVariable(_("Users responsible for this Queue"), 'users', 'set', false, true, null, array($f_users));
        $musers->setDefault($whups_driver->getQueueUsers($queue));
        $userlink = array('text' => _("Edit the users responsible for this queue"), 'url' => Util::addParameter(Horde::applicationUrl('admin/?formname=edituserform'), 'queue', $queue));
        $this->addVariable('', 'link', 'link', false, true, null, array($userlink));

        /* Perms link. */
        if (Auth::isAdmin()) {
            $permslink = array('text' => _("Edit the permissions on this queue"), 'url' => Util::addParameter(Util::addParameter(Horde::url($GLOBALS['registry']->get('webroot', 'horde') . '/admin/perms/edit.php'), 'category', "whups:queues:$queue"), 'autocreate', '1'));
            $this->addVariable('', 'link', 'link', false, true, null, array($permslink));
        }
    }

}

class DeleteQueueForm extends Horde_Form {

    function DeleteQueueForm(&$vars)
    {
        global $whups_driver;

        parent::Horde_Form($vars, _("Delete Queue Confirmation"));

        $queue = $vars->get('queue');
        $info = $whups_driver->getQueue($queue);

        $this->addHidden('', 'queue', 'int', true, true);

        $mname = &$this->addVariable(_("Queue Name"), 'name', 'text', false, true);
        $mname->setDefault($info['name']);

        $mdesc = &$this->addVariable(_("Queue Description"), 'description', 'text', false, true);
        $mdesc->setDefault($info['description']);

        $yesno = array(array(0 => _("No"), 1 => _("Yes")));
        $this->addVariable(_("Really delete this queue? This may cause data problems!"), 'yesno', 'enum', true, false, null, $yesno);
    }

}

///// User Forms

class AddUserForm extends Horde_Form {

    function AddUserForm(&$vars)
    {
        parent::Horde_Form($vars, _("Add Users"));

        $this->addHidden('', 'queue', 'int', true, true);

        global $conf, $whups_driver;

        $auth = &Auth::singleton($conf['auth']['driver']);
        if ($auth->hasCapability('list')) {
            $queue = $vars->get('queue');
            $current = $whups_driver->getQueueUsers($queue);

            $list = $auth->listUsers();
            if (is_a($list, 'PEAR_Error')) {
                $this->addVariable(_("User"), 'user', 'invalid', true, false, null,
                                   array(sprintf(_("There was an error listing users: %s; %s"), $list->getMessage(), $list->getUserInfo())));
            } else {
                sort($list);
                $users = array();
                foreach ($list as $user) {
                    if (!isset($current[$user])) {
                        $users[$user] = Auth::removeHook($user);
                    }
                }
                $this->addVariable(_("User"), 'user', 'multienum', true, false, null, array($users));
            }
        } else {
            $this->addVariable(_("User"), 'user', 'text', true);
        }
    }

}

class EditUserStep1Form extends Horde_Form {

    function EditUserStep1Form(&$vars)
    {
        global $whups_driver;

        $this->appendButtons(_("Remove User"));

        parent::Horde_Form($vars, _("Responsible Users"));

        $queue = $vars->get('queue');
        $users = $whups_driver->getQueueUsers($queue);
        $f_users = array();
        foreach ($users as $user) {
            $f_users[$user] = Whups::formatUser($user);
        }
        if ($f_users) {
            asort($f_users);
            $usertype = 'enum';
            $type_params = array($f_users);
        } else {
            $usertype = 'invalid';
            $type_params = array(_("There are no users responsible for this queue."));
        }

        $this->addHidden('', 'queue', 'int', true, true);
        $this->addVariable(_("Users responsible for this queue"), 'user', $usertype, true, false, null, $type_params);
    }

}

///// Type Forms

class AddTypeStep1Form extends Horde_Form {

    function AddTypeStep1Form(&$vars)
    {
        parent::Horde_Form($vars, _("Add Type"));
        $this->appendButtons(_("Add Type"));
        $this->addVariable(_("Type Name"), 'name', 'text', true);
        $this->addVariable(_("Type Description"), 'description', 'text', true);
    }

}

class EditTypeStep1Form extends Horde_Form {

    function EditTypeStep1Form(&$vars)
    {
        global $whups_driver;

        parent::Horde_Form($vars, _("Edit or Delete Types"));
        $this->setButtons(array(_("Edit Type"), _("Clone Type"), _("Delete Type")));

        $types = $whups_driver->getAllTypes();
        if ($types) {
            $ttype = 'enum';
            $type_params = array($types);
        } else {
            $ttype = 'invalid';
            $type_params = array(_("There are no types to edit"));
        }

        $this->addVariable(_("Type Name"), 'type', $ttype, true, false, null, $type_params);
    }

}

class EditTypeStep2Form extends Horde_Form {

    function EditTypeStep2Form(&$vars)
    {
        global $whups_driver;

        $type = $vars->get('type');
        $info = $whups_driver->getType($type);

        parent::Horde_Form($vars, sprintf(_("Edit %s"), $info['name']));

        $this->addHidden('', 'type', 'int', true, true);

        $tname = &$this->addVariable(_("Type Name"), 'name', 'text', true);
        $tname->setDefault($info['name']);

        $tdesc = &$this->addVariable(_("Type Description"), 'description', 'text', true);
        $tdesc->setDefault($info['description']);

        /* States and states link. */
        $states = $whups_driver->getStates($type);
        $tstates = &$this->addVariable(_("States for this Type"), 'state', 'set', false, true, null, array($states));
        $tstates->setDefault(array_keys($states));
        $statelink = array(array('text' => _("Edit States"), 'url' => Horde::applicationUrl('admin/?formname=editstatestep1form&type=' . $type)));
        if (!count($states)) {
            $statelink[] = array('text' => _("Create Default States"), 'url' => Horde::applicationUrl('admin/?formname=createdefaultstates&type=' . $type));
        }
        $this->addVariable('', 'link', 'link', false, true, null, array($statelink));

        /* Priorities and priorities link. */
        $priorities = $whups_driver->getPriorities($type);
        $tpriorities = &$this->addVariable(_("Priorities for this Type"), 'priority', 'set', false, true, null, array($priorities));
        $tpriorities->setDefault(array_keys($priorities));
        $prioritylink = array(array('text' => _("Edit Priorities"), 'url' => Horde::applicationUrl('admin/?formname=editprioritystep1form&type=' . $type)));
        if (!count($priorities)) {
            $prioritylink[] = array('text' => _("Create Default Priorities"), 'url' => Horde::applicationUrl('admin/?formname=createdefaultpriorities&type=' . $type));
        }
        $this->addVariable('', 'link', 'link', false, true, null, array($prioritylink));

        /* Attributes and attributes link. */
        $attributes = $whups_driver->getAttributesForType($type);
        $params = array();
        foreach ($attributes as $key => $attribute) {
            $params[$key] = $attribute['human_name'];
        }
        $tattributes = &$this->addVariable(_("Attributes for this Type"), 'attribute', 'set', false, true, null, array($params));
        $tattributes->setDefault(array_keys($attributes));
        $attributelink = array('text' => _("Edit Attributes"), 'url' => Horde::applicationUrl('admin/?formname=editattributedescstep1form&type=' . $type));
        $this->addVariable('', 'link', 'link', false, true, null, array($attributelink));
    }

}

class DeleteTypeForm extends Horde_Form {

    function DeleteTypeForm(&$vars)
    {
        global $whups_driver;

        parent::Horde_Form($vars, _("Delete Type Confirmation"));

        $type = $vars->get('type');
        $info = $whups_driver->getType($type);

        $this->addHidden('', 'type', 'int', true, true);

        $tname = &$this->addVariable(_("Type Name"), 'name', 'text', false, true);
        $tname->setDefault($info['name']);

        $tdesc = &$this->addVariable(_("Type Description"), 'description', 'text', false, true);
        $tdesc->setDefault($info['description']);

        $states = $whups_driver->getStates($type);
        $tstates = &$this->addVariable(_("States for this Type"), 'state', 'set', false, true, null, array($states));
        $tstates->setDefault(array_keys($states));

        $priorities = $whups_driver->getPriorities($type);
        $tpriorities = &$this->addVariable(_("Priorities for this Type"), 'priority', 'set', false, true, null, array($priorities));
        $tpriorities->setDefault(array_keys($priorities));

        $yesno = array(array(0 => _("No"), 1 => _("Yes")));
        $this->addVariable(_("Really delete this type? This may cause data problems!"), 'yesno', 'enum', true, false, null, $yesno);
    }

}

class CloneTypeForm extends Horde_Form {

    function CloneTypeForm(&$vars)
    {
        global $whups_driver;

        $type = $vars->get('type');
        $info = $whups_driver->getType($type);
        parent::Horde_Form($vars, sprintf(_("Make a clone of %s"), $info['name']));

        $this->setButtons(_("Clone"));
        $this->addHidden('', 'type', 'int', true, true);

        $tname = &$this->addVariable(_("Name of the cloned copy"), 'name', 'text', true);
        $tname->setDefault(sprintf(_("Copy of %s"), $info['name']));

        $tdesc = &$this->addVariable(_("Clone Description"), 'description', 'text', true);
        $tdesc->setDefault($info['description']);
    }

}

///// Type State Forms

class AddStateForm extends Horde_Form {

    function AddStateForm(&$vars)
    {
        parent::Horde_Form($vars, _("Add State"));

        $this->addHidden('', 'type', 'int', true, true);
        $this->addVariable(_("State Name"), 'name', 'text', true);
        $this->addVariable(_("State Description"), 'description', 'text', true);
        $this->addVariable(_("State Category"), 'category', 'enum', false, false, null, array($GLOBALS['whups_driver']->getCategories()));
    }

}

class EditStateStep1Form extends Horde_Form {

    function EditStateStep1Form(&$vars)
    {
        parent::Horde_Form($vars, _("Edit or Delete States"));
        $this->setButtons(array(_("Edit State"), _("Delete State")));

        $states = $GLOBALS['whups_driver']->getStates($vars->get('type'));
        if ($states) {
            $stype = 'enum';
            $type_params = array($states);
        } else {
            $stype = 'invalid';
            $type_params = array(_("There are no states to edit"));
        }

        $this->addHidden('', 'type', 'int', true, true);
        $this->addVariable(_("State Name"), 'state', $stype, false, false, null, $type_params);
    }

}

class EditStateStep2Form extends Horde_Form {

    function EditStateStep2Form(&$vars)
    {
        parent::Horde_Form($vars, _("Edit State"));

        $state = $vars->get('state');
        $info = $GLOBALS['whups_driver']->getState($state);

        $this->addHidden('', 'type', 'int', true, true);
        $this->addHidden('', 'state', 'int', true, true);

        $sname = &$this->addVariable(_("State Name"), 'name', 'text', true);
        $sname->setDefault($info['name']);

        $sdesc = &$this->addVariable(_("State Description"), 'description', 'text', true);
        $sdesc->setDefault($info['description']);

        $scat = &$this->addVariable(_("State Category"), 'category', 'enum', true, false, null, array($GLOBALS['whups_driver']->getCategories()));
        $scat->setDefault($info['category']);
    }

}

class DefaultStateForm extends Horde_Form {

    function DefaultStateForm(&$vars)
    {
        parent::Horde_Form($vars, _("Set Default State"));

        $this->setButtons(array(_("Set Default State")));

        $states = $GLOBALS['whups_driver']->getStates(
            $vars->get('type'),
            array('unconfirmed', 'new', 'assigned'));
        if ($states) {
            $stype = 'enum';
            $type_params = array($states);
        } else {
            $stype = 'invalid';
            $type_params = array(_("There are no states to edit"));
        }

        $this->addHidden('', 'type', 'int', true, true);
        $var = &$this->addVariable(_("State Name"), 'state', $stype, false,
                                   false, null, $type_params);
        $var->setDefault($GLOBALS['whups_driver']->getDefaultState($vars->get('type')));
    }

}

class DeleteStateForm extends Horde_Form {

    function DeleteStateForm(&$vars)
    {
        parent::Horde_Form($vars, _("Delete State Confirmation"));

        $state = $vars->get('state');
        $info = $GLOBALS['whups_driver']->getState($state);

        $this->addHidden('', 'type', 'int', true, true);
        $this->addHidden('', 'state', 'int', true, true);

        $sname = &$this->addVariable(_("State Name"), 'name', 'text', false, true);
        $sname->setDefault($info['name']);

        $sdesc = &$this->addVariable(_("State Description"), 'description', 'text', false, true);
        $sdesc->setDefault($info['description']);

        $yesno = array(array(0 => _("No"), 1 => _("Yes")));
        $this->addVariable(_("Really delete this state? This may cause data problems!"), 'yesno', 'enum', true, false, null, $yesno);
    }

}

class AddPriorityForm extends Horde_Form {

    function AddPriorityForm(&$vars)
    {
        parent::Horde_Form($vars, _("Add Priority"));

        $this->addHidden('', 'type', 'int', true, true);
        $this->addVariable(_("Priority Name"), 'name', 'text', true);
        $this->addVariable(_("Priority Description"), 'description', 'text', true);
    }

}

class EditPriorityStep1Form extends Horde_Form {

    function EditPriorityStep1Form(&$vars)
    {
        global $whups_driver;

        parent::Horde_Form($vars, _("Edit or Delete Priorities"));
        $this->setButtons(array(_("Edit Priority"), _("Delete Priority")));

        $priorities = $whups_driver->getPriorities($vars->get('type'));
        if ($priorities) {
            $stype = 'enum';
            $type_params = array($priorities);
        } else {
            $stype = 'invalid';
            $type_params = array(_("There are no priorities to edit"));
        }

        $this->addHidden('', 'type', 'int', true, true);
        $this->addVariable(_("Priority Name"), 'priority', $stype, true, false, null, $type_params);
    }

}

class EditPriorityStep2Form extends Horde_Form {

    function EditPriorityStep2Form(&$vars)
    {
        global $whups_driver;

        parent::Horde_Form($vars, _("Edit Priority"));

        $priority = $vars->get('priority');
        $info = $whups_driver->getPriority($priority);

        $this->addHidden('', 'type', 'int', true, true);
        $this->addHidden('', 'priority', 'int', true, true);

        $pname = &$this->addVariable(_("Priority Name"), 'name', 'text', true);
        $pname->setDefault($info['name']);

        $pdesc = &$this->addVariable(_("Priority Description"), 'description', 'text', true);
        $pdesc->setDefault($info['description']);
    }

}

class DefaultPriorityForm extends Horde_Form {

    function DefaultPriorityForm(&$vars)
    {
        parent::Horde_Form($vars, _("Set Default Priority"));

        $this->setButtons(array(_("Set Default Priority")));

        $priorities = $GLOBALS['whups_driver']->getPriorities($vars->get('type'));
        if ($priorities) {
            $stype = 'enum';
            $type_params = array($priorities);
        } else {
            $stype = 'invalid';
            $type_params = array(_("There are no priorities to edit"));
        }

        $this->addHidden('', 'type', 'int', true, true);
        $var = &$this->addVariable(_("Priority Name"), 'priority', $stype, false,
                                   false, null, $type_params);
        $var->setDefault($GLOBALS['whups_driver']->getDefaultPriority($vars->get('type')));
    }

}

class DeletePriorityForm extends Horde_Form {

    function DeletePriorityForm(&$vars)
    {
        global $whups_driver;

        parent::Horde_Form($vars, _("Delete Priority Confirmation"));

        $priority = $vars->get('priority');
        $info = $whups_driver->getPriority($priority);

        $this->addHidden('', 'type', 'int', true, true);
        $this->addHidden('', 'priority', 'int', true, true);

        $pname = &$this->addVariable(_("Priority Name"), 'name', 'text', false, true);
        $pname->setDefault($info['name']);

        $pdesc = &$this->addVariable(_("Priority Description"), 'description', 'text', false, true);
        $pdesc->setDefault($info['description']);

        $yesno = array(array(0 => _("No"), 1 => _("Yes")));
        $this->addVariable(_("Really delete this priority? This may cause data problems!"), 'yesno', 'enum', true, false, null, $yesno);
    }

}

class AddVersionForm extends Horde_Form {

    function AddVersionForm(&$vars)
    {
        parent::Horde_Form($vars, _("Add Version"));

        $this->addHidden('', 'queue', 'int', true, true);
        $this->addVariable(_("Version Name"), 'name', 'text', true);
        $this->addVariable(_("Version Description"), 'description', 'text', true);
    }

}

class EditVersionStep1Form extends Horde_Form {

    function EditVersionStep1Form(&$vars)
    {
        global $whups_driver;

        parent::Horde_Form($vars, _("Edit or Delete Versions"));
        $this->setButtons(array(_("Edit Version"), _("Delete Version")));

        $versions = $whups_driver->getVersions($vars->get('queue'));
        if ($versions) {
            $vtype = 'enum';
            $type_params = array($versions);
        } else {
            $vtype = 'invalid';
            $type_params = array(_("There are no versions to edit"));
        }

        $this->addHidden('', 'queue', 'int', true, true);
        $this->addVariable(_("Version Name"), 'version', $vtype, true, false, null, $type_params);
    }

}

class EditVersionStep2Form extends Horde_Form {

    function EditVersionStep2Form(&$vars)
    {
        global $whups_driver;

        parent::Horde_Form($vars, _("Edit Version"));

        $version = $vars->get('version');
        $info = $whups_driver->getVersion($version);

        $this->addHidden('', 'queue', 'int', true, true);
        $this->addHidden('', 'version', 'int', true, true);

        $vname = &$this->addVariable(_("Version Name"), 'name', 'text', true);
        $vname->setDefault($info['name']);

        $vdesc = &$this->addVariable(_("Version Description"), 'description', 'text', true);
        $vdesc->setDefault($info['description']);
    }

}

class DeleteVersionForm extends Horde_Form {

    function DeleteVersionForm(&$vars)
    {
        global $whups_driver;

        parent::Horde_Form($vars, _("Delete Version Confirmation"));

        $version = $vars->get('version');
        $info = $whups_driver->getVersion($version);

        $this->addHidden('', 'queue', 'int', true, true);
        $this->addHidden('', 'version', 'int', true, true);

        $vname = &$this->addVariable(_("Version Name"), 'name', 'text', false, true);
        $vname->setDefault($info['name']);

        $vdesc = &$this->addVariable(_("Version Description"), 'description', 'text', false, true);
        $vdesc->setDefault($info['description']);

        $yesno = array(array(0 => _("No"), 1 => _("Yes")));
        $this->addVariable(_("Really delete this version? This may cause data problems!"), 'yesno', 'enum', true, false, null, $yesno);
    }

}

///// Type Attribute Description Forms

class AddAttributeDescForm extends Horde_Form {

    function AddAttributeDescForm(&$vars)
    {
        parent::Horde_Form($vars, _("Add Attribute"));

        $this->addHidden('', 'type', 'int', true, true);
        $this->addVariable(_("Attribute Name"), 'name', 'text', true);
        $this->addVariable(_("Attribute Description"), 'description', 'text', true);
    }

}

class EditAttributeDescStep1Form extends Horde_Form {

    function EditAttributeDescStep1Form(&$vars)
    {
        global $whups_driver;

        parent::Horde_Form($vars, _("Edit or Delete Attributes"));
        $this->setButtons(array(_("Edit Attribute"), _("Delete Attribute")));

        $attributes = $whups_driver->getAttributesForType($vars->get('type'));
        if ($attributes) {
            $params = array();
            foreach ($attributes as $key => $attribute) {
                $params[$key] = $attribute['human_name'];
            }
            $stype = 'enum';
            $type_params = array($params);
        } else {
            $stype = 'invalid';
            $type_params = array(_("There are no attribute types to edit"));
        }

        $this->addHidden('', 'type', 'int', true, true);
        $this->addVariable(_("Attribute Name"), 'attribute', $stype, true, false, null, $type_params);
    }

}

class EditAttributeDescStep2Form extends Horde_Form {

    function EditAttributeDescStep2Form(&$vars)
    {
        global $whups_driver;

        parent::Horde_Form($vars, _("Edit Attribute"));

        $attribute = $vars->get('attribute');
        $info = $whups_driver->getAttributeDesc($attribute);

        $this->addHidden('', 'type', 'int', true, true);
        $this->addHidden('', 'attribute', 'int', true, true);

        $pname = &$this->addVariable(_("Attribute Name"), 'name', 'text', true);
        $pname->setDefault($info['attribute_name']);

        $pdesc = &$this->addVariable(_("Attribute Description"), 'description', 'text', true);
        $pdesc->setDefault($info['attribute_description']);
    }

}

class DeleteAttributeDescForm extends Horde_Form {

    function DeleteAttributeDescForm(&$vars)
    {
        global $whups_driver;

        parent::Horde_Form($vars, _("Delete Attribute Confirmation"));

        $attribute = $vars->get('attribute');
        $info = $whups_driver->getAttributeDesc($attribute);

        $this->addHidden('', 'type', 'int', true, true);
        $this->addHidden('', 'attribute', 'int', true, true);

        $pname = &$this->addVariable(_("Attribute Name"), 'name', 'text', false, true);
        $pname->setDefault($info['attribute_name']);

        $pdesc = &$this->addVariable(_("Attribute Description"), 'description', 'text', false, true);
        $pdesc->setDefault($info['attribute_description']);

        $yesno = array(array(0 => _("No"), 1 => _("Yes")));
        $this->addVariable(_("Really delete this attribute? This may cause data problems!"), 'yesno', 'enum', true, false, null, $yesno);
    }

}

class SendReminderForm extends Horde_Form {

    function SendReminderForm(&$vars)
    {
        global $whups_driver;

        parent::Horde_Form($vars, _("Send Reminders"));

        $queues = Whups::permissionsFilter($whups_driver->getQueues(), 'queue', PERMS_EDIT);
        if (count($queues)) {
            $modtype = 'enum';
            $type_params = array($queues);
        } else {
            $modtype = 'invalid';
            $type_params = array(_("There are no queues available."));
        }

        $this->addVariable(_("Send only for this list of ticket ids"), 'id', 'intlist', false);
        $this->addVariable(_("For tickets from these queues"), 'queue', $modtype, false, false, null, $type_params);

        $cats = $whups_driver->getCategories();
        unset($cats['resolved']);
        $categories = &$this->addVariable(_("For tickets which are"), 'category', 'multienum', false, false, null, array($cats, 3));
        $categories->setDefault(array('assigned'));

        $this->addVariable(_("Unassigned tickets"), 'unassigned', 'email', false, false, _("If you select any tickets that do not have an owner, who should we send email to?"));
    }

}
