<?php
/**
 * $Horde: whups/lib/Renderer/Comment.php,v 1.5 2008/02/28 05:01:45 chuck Exp $
 *
 * Copyright 2001-2007 Robert E. Coyle <robertecoyle@hotmail.com>
 *
 * See the enclosed file LICENSE for license information (BSD). If you
 * did not receive this file, see http://www.horde.org/licenses/bsdl.php.
 *
 * @package Whups
 */
class Horde_Form_Renderer_Comment extends Horde_Form_Renderer {

    var $queue = null;
    var $type = null;
    var $state = null;
    var $priority = null;
    var $due = null;

    function begin($title)
    {
        $this->_sectionHeader($title);
        echo '<div id="comments">';
    }

    function render($transaction, &$vars)
    {
        global $prefs, $conf, $registry;

        static $canUpdate;
        if (!isset($canUpdate)) {
            $canUpdate = Auth::getAuth() && Whups::hasPermission(array($vars->get('queue')), 'queue', 'update', Auth::getAuth());
        }

        $comment = '';
        $private = false;
        $changes = array();

        $changelist = $vars->get('changes');
        if (!$changelist) {
            return '';
        }

        /* Format each change in this history entry, including
         * comments, etc. */
        foreach ($changelist as $change) {
            switch ($change['type']) {
            case 'summary':
                $changes[] = sprintf(_("Summary &rArr; %s"), htmlspecialchars($change['value']));
                break;

            case 'attachment':
                $ticket = $vars->get('ticket_id');
                if ($file = Whups::getAttachments($ticket, $change['value'])) {
                    $changes[] = sprintf(_("New Attachment: %s"), Whups::attachmentUrl($ticket, $file));
                } else {
                    $changes[] = sprintf(_("New Attachment: %s"), htmlspecialchars($change['value']));
                }
                break;

            case 'delete-attachment':
                $changes[] = sprintf(_("Deleted Attachment: %s"), htmlspecialchars($change['value']));
                break;

            case 'assign':
                $changes[] = sprintf(_("Assigned to %s"),
                                     Whups::formatUser($change['value'], false, true, true));
                break;

            case 'unassign':
                $changes[] = sprintf(_("Taken from %s"),
                                     Whups::formatUser($change['value'], false, true, true));
                break;

            case 'comment':
                $comment = $change['comment'];
                $private = !empty($change['private']);
                if ($comment) {
                    $reply = Horde::link(Horde::applicationUrl(
                                 Util::addParameter($canUpdate ? 'ticket/update.php' : 'ticket/comment.php',
                                                    array('id' => $vars->get('ticket_id'),
                                                          'transaction' => $transaction)))) .
                        _("Reply to this comment") . '</a>';
                }
                break;

            case 'queue':
                $changes[] = sprintf(_("Queue &rArr; %s"), htmlspecialchars($change['label']));
                break;

            case 'type':
                $changes[] = sprintf(_("Type &rArr; %s"), htmlspecialchars($change['label']));
                break;

            case 'state':
                $changes[] = sprintf(_("State &rArr; %s"), htmlspecialchars($change['label']));
                break;

            case 'priority':
                $changes[] = sprintf(_("Priority &rArr; %s"), htmlspecialchars($change['label']));
                break;

            case 'attribute':
                $changes[] = sprintf(_("%s &rArr; %s"), htmlspecialchars($change['label']), htmlspecialchars($change['value']));
                break;

            case 'due':
                if ($change['label']) {
                    $changes[] = sprintf(_("Due &rArr; %s"), strftime($prefs->getValue('date_format'), $change['label']));
                }
                break;
            }
        }

        if ($comment) {
            $comment = Text_Filter::filter($comment, 'text2html', array('parselevel' => TEXT_HTML_MICRO, 'class' => null));
            if ($prefs->getValue('autolink_tickets') &&
                $conf['prefs']['autolink_terms']) {
                $term_regex = '/(' . $conf['prefs']['autolink_terms'] . ')\s*#?(\d+)/i';
                $comment = preg_replace_callback($term_regex, array(&$this, '_autolink'), $comment);
            }

            if (!count($changes)) {
                $changes[] = _("Comment");
            }
        }

        if (count($changes)) {
            ob_start();
            $class = $private ? 'pc' : 'c';
            $icon = $private ? Horde::img('locked.png', '', null, $registry->getImageDir('horde')) : '';
?>
<div id="t<?php echo (int)$transaction ?>">
<table cellspacing="0" width="100%">
 <tr>
  <td width="20%" class="<?php echo $class ?>_l nowrap" valign="top"><?php echo strftime($prefs->getValue('date_format'), $vars->get('timestamp')) ?></td>
  <td width="20%" class="<?php echo $class ?>_m" valign="top"><?php echo $vars->get('user_id') ? Whups::formatUser($vars->get('user_id'), false, true, true) : '&nbsp;' ?></td>
  <td width="30%" class="<?php echo $class ?>_m" valign="top"><?php echo implode('<br />', $changes) ?></td>
  <td width="30%" class="<?php echo $class ?>_r rightAlign" valign="top"><?php if ($comment) echo $reply ?></td>
 </tr>
<?php if ($comment): ?>
 <tr><td colspan="4" class="<?php echo $class ?>_b">
  <div class="comment-body">
<?php if (!empty($icon)): ?>
   <span class="leftFloat"><?php echo $icon ?></span>
<?php endif; ?>
   <code><?php echo $comment ?></code>
  </div>
 </td></tr>
<?php else: ?>
 <tr><td colspan="4" class="c_b">&nbsp;</td></tr>
<?php endif; ?>
</table>
</div>
<?php
            $html = ob_get_contents();
            ob_end_clean();
            return $html;
        }

        return '';
    }

    function _autolink($matches)
    {
        $url = Whups::urlFor('ticket', $matches[2]);
        $linktext = $matches[1] . ' ' . $matches[2];
        $link = '<strong>' . Horde::link($url, 'View ' . $linktext) . $linktext . '</a></strong>';
        $details = $GLOBALS['whups_driver']->getTicketDetails($matches[2]);
        if (!is_a($details, 'PEAR_Error') &&
            $details['state_category'] == 'resolved') {
            $link = '<del>' . $link . '</del>';
        }
        return $link;
    }

    function end()
    {
        echo '</div>';
    }

}
